/*
 * Java Genetic Algorithm Library (jenetics-1.6.0).
 * Copyright (c) 2007-2014 Franz Wilhelmstötter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author:
 *    Franz Wilhelmstötter (franz.wilhelmstoetter@gmx.at)
 */
package org.jenetics;

/**
 * Abstract base class for implementing concrete NumericGenes.
 *
 * @author <a href="mailto:franz.wilhelmstoetter@gmx.at">Franz Wilhelmstötter</a>
 * @version 1.6 &mdash; <em>$Date: 2014-03-05 $</em>
 * @since 1.6
 */
abstract class AbstractNumericGene<
	N extends Number & Comparable<? super N>,
	G extends AbstractNumericGene<N, G>
>
	extends AbstractBoundedGene<N, G>
	implements NumericGene<N, G>
{
	private static final long serialVersionUID = 1L;

	/**
	 * Create new {@code NumericGene}.
	 *
	 * @param value The value of the gene.
	 * @param min The allowed min value of the gene.
	 * @param max The allows max value of the gene.
	 * @throws NullPointerException if one of the given arguments is
	 *         {@code null}.
	 */
	protected AbstractNumericGene(final N value, final N min, final N max) {
		super(value, min, max);
	}

	@Override
	public byte byteValue() {
		return _value.byteValue();
	}

	@Override
	public short shortValue() {
		return _value.shortValue();
	}

	@Override
	public int intValue() {
		return _value.intValue();
	}

	@Override
	public long longValue() {
		return _value.longValue();
	}

	@Override
	public float floatValue() {
		return _value.floatValue();
	}

	@Override
	public double doubleValue() {
		return _value.doubleValue();
	}

	@Override
	public abstract G newInstance(final Number number);

}
